@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function, unicode_literals, with_statement
-from subprocess import Popen, STDOUT, PIPE
+from subprocess import Popen, PIPE
 
 try:
     from urllib import unquote
@@ -8,8 +8,8 @@ except ImportError:  # python 3
     from urllib.parse import unquote
 
 import re
-import os.path
 import os
+import sys
 
 try:
     import json  # for reading logs
@@ -44,7 +44,11 @@ class Revision(object):
         rev = json.loads(json_log)
 
         for key in rev.keys():
-            self.__setattr__(key, unquote(rev[key]))
+            if sys.version_info.major < 3:
+                _value = unquote(rev[key].encode("ascii")).decode("utf-8")
+            else:
+                _value = unquote(rev[key])
+            self.__setattr__(key, _value)
         self.rev = int(self.rev)
         if not self.branch:
             self.branch = 'default'
@@ -80,17 +84,18 @@ class Repo(object):
 
             Raise on error.
         """
-        proc = Popen(["hg", "--cwd", path, "--encoding", "UTF-8"] + list(args),
+        cmd = ["hg", "--cwd", path, "--encoding", "UTF-8"] + list(args)
+        proc = Popen(cmd,
                      stdout=PIPE, stderr=PIPE, env=env)
 
         out, err = [x.decode("utf-8") for x in proc.communicate()]
 
         if proc.returncode:
-            cmd = (" ".join(["hg", "--cwd", path] + list(args)))
-            raise HgException("Error running %s:\n\" + "
-                              "tErr: %s\n\t"
-                              "Out: %s\n\t"
-                              "Exit: %s"
+            cmd = " ".join(cmd)
+            raise HgException("Error running %s:\n"
+                              "\tErr: %s\n"
+                              "\tOut: %s\n"
+                              "\tExit: %s"
                               % (cmd, err, out, proc.returncode),
                               exit_code=proc.returncode)
 
@@ -191,7 +196,7 @@ class Repo(object):
         """
             Get all tags from the repo.
 
-            Retruns a dict containing tag: shortnode mapping
+            Returns a dict containing tag: shortnode mapping
         """
         cmd = ['tags']
         output = self.hg_command(*cmd)
@@ -276,6 +281,38 @@ class Repo(object):
         else:
             self.hg_command("pull", source)
 
+    def hg_paths(self):
+        """Get remote repositories."""
+        remotes = self.hg_command("paths").split("\n")
+        remotes_list = [line.split(" = ") for line in remotes if line != ""]
+
+        return dict(remotes_list)
+
+    def __get_remote_changes(self, command, remote):
+        if remote not in self.hg_paths().keys():
+            raise HgException("No such remote repository")
+
+        try:
+            result = self.hg_command(
+                command,
+                remote,
+                "--template",
+                self.rev_log_tpl
+            ).split("\n")
+        except HgException:
+            return []
+
+        changesets = [change for change in result if change.startswith("{")]
+        return list(map(lambda revision: Revision(revision), changesets))
+
+    def hg_outgoing(self, remote="default"):
+        """Get outgoing changesets for a certain remote."""
+        return self.__get_remote_changes("outgoing", remote)
+
+    def hg_incoming(self, remote="default"):
+        """Get incoming changesets for a certain remote."""
+        return self.__get_remote_changes("incoming", remote)
+
     def hg_log(self, identifier=None, limit=None, template=None,
                branch=None, **kwargs):
         """Get repositiory log."""
@@ -308,17 +345,19 @@ class Repo(object):
         return branch.strip()
 
     def get_branches(self):
-        """ Returns a list of branches from the repo, including versions."""
+        """
+            Returns a list of branches from the repo, including versions.
+
+            If get_active_only is True, then only return active branches.
+        """
         branches = self.hg_command("branches")
         branch_list = branches.strip().split("\n")
         values = []
         for branch in branch_list:
-            b = branch.partition(" ")
+            b = re.split('(\d+:[A-Za-z0-9]+)', branch)
             if not b:
                 continue
-            name = b[0].strip()
-            version = b[-1].strip()
-            values.append({'name': name, 'version': version})
+            values.append({'name': b[0].strip(), 'version': b[1].strip()})
         return values
 
     def get_branch_names(self):
@@ -327,7 +366,7 @@ class Repo(object):
         branch_list = branches.strip().split("\n")
         values = []
         for branch in branch_list:
-            b = branch.partition(" ")
+            b = re.split('(\d+:[A-Za-z0-9]+)', branch)
             if not b:
                 continue
             name = b[0]
@@ -336,6 +375,52 @@ class Repo(object):
                 values.append(name)
         return values
 
+    BOOKMARK_LIST = 0
+    BOOKMARK_CREATE = 1
+    BOOKMARK_DELETE = 2
+    BOOKMARK_RENAME = 3
+    BOOKMARK_INACTIVE = 4
+
+    def hg_bookmarks(self, action=BOOKMARK_LIST, name=None, newname=None,
+                     revision=None, force=False):
+        cmds = ['bookmarks']
+        if force:
+            cmds += ['--force']
+        if revision:
+            cmds += ['--rev', str(revision)]
+        if action == Repo.BOOKMARK_LIST:
+            out = self.hg_command(*cmds)
+            bookmarks = []
+            if out.startswith(" "):  # handles "no bookmarks set" reply
+                for line in out.split('\n'):
+                    if line:
+                        # active/inactive
+                        if line.strip()[0] == '*':
+                            bookmark = [True]
+                            line = line[3:]
+                        else:
+                            bookmark = [False]
+                        # name and identifier
+                        line.split()
+                        bookmark += [line.split()[0].strip(), line.split()[1]]
+                        bookmarks += [bookmark]
+            return bookmarks
+        elif action == Repo.BOOKMARK_INACTIVE:
+            cmds += ['--inactive']
+            if name:
+                cmds += [name]
+            return self.hg_command(*cmds)
+        elif name is not None:
+            if action == Repo.BOOKMARK_DELETE:
+                cmds += ['--delete', name]
+                return self.hg_command(*cmds)
+            elif action == Repo.BOOKMARK_RENAME and newname is not None:
+                cmds += ['--rename', name, newname]
+                return self.hg_command(*cmds)
+            elif action == Repo.BOOKMARK_CREATE:
+                cmds += [name]
+                return self.hg_command(*cmds)
+
     def hg_diff(self, rev_a=None, rev_b=None, filenames=None):
         """
             Get a unified diff as returned by 'hg diff'.
@@ -349,10 +434,10 @@ class Repo(object):
         """
         cmds = ['diff']
         for rev in (rev_a, rev_b):
-            if not rev is None:
+            if rev is not None:
                 cmds += ['-r', rev]
 
-        if not filenames is None:
+        if filenames is not None:
             cmds += list(filenames)
 
         result = self.hg_command(*cmds)
@@ -418,7 +503,7 @@ class Repo(object):
         return Revision(out)
 
     def revisions(self, slice_):
-        """Retruns a list of Revision objects for the given slice"""
+        """Returns a list of Revision objects for the given slice"""
         id = ":".join([str(x) for x in (slice_.start, slice_.stop)])
         out = self.hg_log(identifier=id,
                           template=self.rev_log_tpl)
