@@ -1,12 +1,10 @@
-from __future__ import with_statement
+#  -*- encoding: utf-8 -*-
+from __future__ import with_statement, unicode_literals
 
 import unittest
 import doctest
-
 import os
-
 import shutil
-
 import hgapi
 
 
@@ -305,7 +303,27 @@ class TestHgAPI(unittest.TestCase):
         branch_names = self.repo.get_branch_names()
         self.assertEquals(len(branch_names), 2)
 
+        # Test branch name with space
+        branch = self.repo.hg_branch('test branch with space')
+        message = "marked working directory as branch test branch with space"
+        self.assertTrue(branch.startswith(message))
+        self.assertEquals(self.repo.hg_branch(), "test branch with space")
+        self.repo.hg_commit("commit test branch with space")
+        self.assertEquals(self.repo.hg_branch(), "test branch with space")
+        branches = self.repo.get_branches()
+        self.assertEquals(len(branches), 3)
+        branch_names = self.repo.get_branch_names()
+        self.assertEquals(len(branch_names), 3)
+
+        # Test closing of a branch
+        self.repo.hg_commit("Closing test branch", close_branch=True)
+        branches = self.repo.get_branches()
+        self.assertEquals(len(branches), 2)
+        branch_names = self.repo.get_branch_names()
+        self.assertEquals(len(branch_names), 2)
+
     def test_200_CommitWithDates(self):
+        self.repo.hg_update("test_branch")
         rev0 = self.repo.hg_rev()
 
         with open("test/file.txt", "w+") as out:
@@ -337,10 +355,6 @@ class TestHgAPI(unittest.TestCase):
     def test_220_LogWithBranch(self):
         default = self.repo.hg_log(branch='default')
         branch = self.repo.hg_log(branch='test_branch')
-
-        print(default)
-        print(branch)
-
         self.assertTrue("commit test_branch" in branch)
         self.assertFalse("commit test_branch" in default)
 
@@ -441,15 +455,111 @@ class TestHgAPI(unittest.TestCase):
         # non existing repo
         self.assertRaises(hgapi.HgException, hgapi.Repo.hg_root, "./whatever")
 
+    def test_411_paths(self):
+        paths = self.repo.hg_paths()
+        self.assertEquals(paths, {})
+
+        paths = self.clone.hg_paths()
+        self.assertNotEquals(paths, {})
+
+        self.assertTrue("default" in paths)
+        self.assertTrue(paths['default'].endswith('test'))
+
+    def test_412_outgoing(self):
+        # modify file in the cloned repository and commit it
+        with open("./test-clone/cities/ghent.txt", "a") as out:
+            out.write("amsterdam")
+        self.clone.hg_commit("[CLONE] Modified file")
+
+        outgoing = self.clone.hg_outgoing()
+        self.assertEquals(1, len(outgoing))
+        self.assertEquals("[CLONE] Modified file", outgoing[0].desc)
+
+        # push and check outgoing changes again
+        self.clone.hg_push()
+        outgoing = self.clone.hg_outgoing()
+        self.assertEquals(0, len(outgoing))
+
+        # a repository without remote should throw an exception
+        self.assertRaises(hgapi.HgException, self.repo.hg_outgoing)
+
+    def test_413_incoming(self):
+        with open("./test/cities/ghent.txt", "a") as out:
+            out.write("amstelveen")
+        self.repo.hg_commit("[CLONE] Modified file again")
+
+        incoming = self.clone.hg_incoming()
+        self.assertEquals(1, len(incoming))
+        self.assertEquals("[CLONE] Modified file again", incoming[0].desc)
+
+        # pull changes, update and check incoming again
+        self.clone.hg_pull()
+        self.clone.hg_update("tip")
+
+        incoming = self.clone.hg_incoming()
+        self.assertEquals(0, len(incoming))
+
+        # a repository without remote should throw an exception
+        self.assertRaises(hgapi.HgException, self.repo.hg_incoming)
+
+    def test_420_CommitWithNonAsciiCharacters(self):
+        with open("test/file3.txt", "w") as out:
+            out.write("enjoy a new file")
+        self.repo.hg_add("file3.txt")
+
+        self.repo.hg_commit("éàô",
+                            user="F. Håård",
+                            date="10/10/11 UTC")
+
+        rev = self.repo["tip"]
+
+        self.assertEquals(rev.desc, "éàô")
+        self.assertEquals(rev.author, "F. Håård")
+
+    def test_430_Bookmarks(self):
+        # check no bookmarks
+        self.assertListEqual(self.repo.hg_bookmarks(), [])
+        empty_list = self.repo.hg_bookmarks(action=self.repo.BOOKMARK_LIST)
+        self.assertListEqual(empty_list, [])
+        # create bookmark at tip (revision 23:somevalue)
+        self.repo.hg_bookmarks(action=self.repo.BOOKMARK_CREATE,
+                               name="foo")
+        # [True, 'foo', '23:somevalue']
+        self.assertTrue(self.repo.hg_bookmarks()[0][0])
+        self.assertEqual(self.repo.hg_bookmarks()[0][1], 'foo')
+        self.assertTrue('25:' in self.repo.hg_bookmarks()[0][2])
+        # create bookmark at '10:somevalue' named 'bar'
+        self.repo.hg_bookmarks(action=self.repo.BOOKMARK_CREATE,
+                               name="bar", revision=10)
+        self.assertFalse(self.repo.hg_bookmarks()[0][0])
+        self.assertEqual(self.repo.hg_bookmarks()[0][1], 'bar')
+        self.assertTrue('10:' in self.repo.hg_bookmarks()[0][2])
+        # rename foo to fizz
+        self.repo.hg_bookmarks(action=self.repo.BOOKMARK_RENAME,
+                               name='foo', newname='fizz')
+        self.assertTrue(self.repo.hg_bookmarks()[1][0])
+        self.assertEqual(self.repo.hg_bookmarks()[1][1], 'fizz')
+        self.assertTrue('25:' in self.repo.hg_bookmarks()[1][2])
+        # make fizz inactive
+        self.repo.hg_bookmarks(action=self.repo.BOOKMARK_INACTIVE,
+                               name='fizz')
+        self.assertFalse(self.repo.hg_bookmarks()[1][0])
+        self.assertEqual(self.repo.hg_bookmarks()[1][1], 'fizz')
+        self.assertTrue('25:' in self.repo.hg_bookmarks()[1][2])
+        # delete fizz
+        self.repo.hg_bookmarks(action=self.repo.BOOKMARK_DELETE,
+                               name='fizz')
+        self.assertTrue(len(self.repo.hg_bookmarks()) == 1)
+
 
 def test_doc():
     # prepare for doctest
     os.mkdir("./test_hgapi")
     with open("test_hgapi/file.txt", "w") as target:
-        w = target.write("stuff")
+        target.write("stuff")
     try:
         # run doctest
-        res = doctest.testfile("../README.rst")
+        doctest.testfile("../README.rst")
     finally:
         # cleanup
         shutil.rmtree("test_hgapi")
